<?php

/**
 * WC_BrazilPays_Gateway_CreditNew
 *
 * Providencia um Gateway de pagamento próprio via Cartão de Crédito do BrazilPays
 *
 * @class       WC_BrazilPays_Gateway_CreditNew
 * @extends     WC_Payment_Gateway
 * @version     2.0.0
 * @package     WooCommerce\Classes\Payment
 */

 require_once plugin_dir_path(__FILE__) . 'class-brazilpays-environment.php';

class WC_BrazilPays_Gateway_CreditNew extends WC_Payment_Gateway
{
    /**
     * Gateway instructions that will be added to the thank you page and emails.
     *
     * @var string
     */
    public $instructions;

    /**
     * API URL
     *
     * @var string
     */
    private $api_url;

    protected $merchant_code;
    protected $public_key;
    protected $auth_token;
    public $description;
    protected $exchange_rate;
    protected $calculated_usd_value;
    

    /**
     * Constructor for the gateway.
     */
    public function __construct()
    {
        $this->id = 'brazilpays-credit';
        $this->icon = apply_filters('brazilpays-plugin', plugins_url('../assets/icon-credit.png', __FILE__));
        $this->has_fields = true;
        $this->method_title = __('Cartão de Crédito - Brazil Pays Pagamentos', 'brazilpays-plugin');
        $this->method_description = __('Receba pagamentos no crédito utilizando sua conta BrazilPays', 'brazilpays-plugin');

        // Load the settings.
        $this->init_form_fields();
        $this->init_settings();

        // Define user set variables
        $this->title = $this->get_option('title');
        $this->description = $this->get_option('description');
        $this->instructions = $this->get_option('instructions', $this->description);
        
         // Obtem as configurações do BrazilPaysEnvironment
       $brazilpays_environment = BrazilPaysEnvironment::getInstance();
        if ($brazilpays_environment) {
            $this->merchant_code = $brazilpays_environment->getMerchantCode();
            $this->public_key = $brazilpays_environment->getPublicKey();
            $this->api_url = $brazilpays_environment->getApiUrl();
        } else {
            // Log an error or handle the case where BrazilPaysEnvironment is not initialized
            error_log('BrazilPaysEnvironment not initialized');
        }
       

        // Actions
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        add_action('woocommerce_thankyou_' . $this->id, array($this, 'thankyou_page'));
        add_action('woocommerce_email_before_order_table', array($this, 'email_instructions'), 10, 3);
        //add_action('woocommerce_checkout_process', array($this, 'validate_fields'));
        add_action('init', array($this, 'register_custom_order_meta'));


        // HPOS compatibility
        add_action('before_woocommerce_init', array($this, 'declare_hpos_compatibility'));

        // Set API URL
        //$this->api_url = BrazilPaysEnvironment::getInstance()->getApiUrl();

        $this->auth_token = $this->get_auth_token();
    }


    private function get_auth_token()
    {
        // Check for cached token
        $cached_token = get_transient('brazilpays_token');
        $token_expiry = get_transient('brazilpays_token_expiry');

        if ($cached_token && $token_expiry && time() < $token_expiry) {
            error_log('BrazilPays: Using cached token');
            return $cached_token;
        }

        // Ensure we have the merchant code and public key
        $brazilpays_environment = BrazilPaysEnvironment::getInstance();
       $this->merchant_code = $brazilpays_environment->getMerchantCode();
       $this->public_key = $brazilpays_environment->getPublicKey();

        if (empty($this->merchant_code) || empty($this->public_key)) {
            error_log('BrazilPays: Missing merchant code or public key');
            return null;
        }

        error_log('BrazilPays: Attempting to get new token');
        error_log('BrazilPays: Merchant Code: ' . substr($this->merchant_code, 0, 5) . '...');
        error_log('BrazilPays: Public Key: ' . substr($this->public_key, 0, 5) . '...');

        $response = wp_remote_post($this->api_url . '/Merchant/External/Token', [
            'headers' => [
                'merchantCode' => $this->merchant_code,
                'publicKey' => $this->public_key,
                'Content-Type' => 'application/json'
            ],
            'body' => json_encode([
                'merchantCode' => $this->merchant_code,
                'publicKey' => $this->public_key
            ]),
            'timeout' => 30
        ]);

        if (is_wp_error($response)) {
            error_log('BrazilPays: WP Error in token request - ' . $response->get_error_message());
            return null;
        }

        $response_code = wp_remote_retrieve_response_code($response);
        $body = wp_remote_retrieve_body($response);

        error_log('BrazilPays: Token API Response Code: ' . $response_code);
        error_log('BrazilPays: Token API Response Body: ' . $body);

        if ($response_code !== 200) {
            error_log('BrazilPays: Failed to get token. Response code: ' . $response_code);
            return null;
        }

        $data = json_decode($body, true);
        $token = $data['data']['access_token'] ?? null;
        $expires_in = $data['data']['expires_in'] ?? 3600;

        if ($token) {
            set_transient('brazilpays_token', $token, $expires_in);
            set_transient('brazilpays_token_expiry', time() + $expires_in, $expires_in);
            error_log('BrazilPays: Successfully obtained and cached new token');
        } else {
            error_log('BrazilPays: Failed to extract token from API response');
        }

        return $token;
    }

    private function get_exchange_rate()
    {
        if (!$this->auth_token) {
            return null;
        }

        $response = wp_remote_post($this->api_url . '/Charge/Calculate', [
            'headers' => [
                'Authorization' => 'Bearer ' . $this->auth_token,
                'Content-Type' => 'application/json'
            ],
            'body' => json_encode(['amount' => 1.00]),
            'timeout' => 30
        ]);

        if (is_wp_error($response) || wp_remote_retrieve_response_code($response) !== 200) {
            return null;
        }

        $body = wp_remote_retrieve_body($response);
        $data = json_decode($body, true);

        return [
            'cotacao' => $data['data']['exchange'] ?? null,
            'usedExchange' => $data['data']['usedExchange'] ?? null
        ];
    }



    /**
     * Declare HPOS compatibility
     */
    public function declare_hpos_compatibility()
    {
        if (class_exists('\Automattic\WooCommerce\Utilities\FeaturesUtil')) {
            \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility('custom_order_tables', __FILE__, true);
        }
    }

    /**
     * Register custom order meta data for HPOS compatibility
     */
    public function register_custom_order_meta()
    {
        $meta_keys = array(
            'id_transacao',
            'card_cpf_cnpj',
            'card_number',
            'card_name',
            'card_month',
            'card_year',
            'card_cvv',
            'card_gender',
            'card_birth_date',
            'card_installments'
        );

        foreach ($meta_keys as $meta_key) {
            register_post_meta('shop_order', $meta_key, array(
                'show_in_rest' => true,
                'single' => true,
                'type' => 'string',
                'auth_callback' => function () {
                    return current_user_can('edit_shop_orders');
                }
            ));
        }
    }

    /**
     * Process the payment and return the result
     *
     * @param int $order_id
     * @return array
     */
    public function process_payment($order_id)
    {

        // Primeiro, validamos os campos
        $validation_result = $this->validate_fields();
        if (!$validation_result) {
            // Se a validação falhar, retornamos falha
            return array('result' => 'fail');
        }
        
        $order = wc_get_order($order_id);
        
        // Use métodos compatíveis com HPOS
        $order->set_payment_method($this->id);
        $order->set_payment_method_title($this->get_title());
        
        // Obtenha o token atualizado
        $token = $this->authToken();
        
        // Verificar se o token foi recuperado com sucesso
        if (empty($token)) {
            wc_add_notice(__('Ocorreu um erro ao obter o token de autenticação!', 'brazilpays-plugin'), 'error');
            return ['result' => 'fail'];
        }
        $this->exchange_rate = $this->auth_token ? $this->get_exchange_rate($this->auth_token) : null;
        $cotacao_dolar = $this->exchange_rate;

        if (empty($token) || empty($cotacao_dolar)) {
            return ['result' => 'fail'];
        }

        $cart_total = $order->get_total();
        $total = (string)$cart_total;

        if ($cart_total < 1.00) {
            wc_add_notice(
                __('O valor total do pedido deve ser superior a $1.00!', 'brazilpays-plugin'),
                'error'
            );
            return ['result' => 'fail'];
        }


        // Preparar dados do pedido
        $body_req = $this->prepare_order_data($order, $total, $cotacao_dolar);

        $logger = wc_get_logger();
        $logger->info('Dados enviados para API: ' . print_r($body_req, true), array('source' => 'brazilpays-payment-process'));

        
        $response = $this->api_request('POST', '/Charge', $body_req);

        if (is_wp_error($response)) {
            wc_add_notice($response->get_error_message(), 'error');
            return array('result' => 'fail');
        }

        if (!is_wp_error($response)) {
            $body = wp_remote_retrieve_body($response);
            $data = json_decode($body, true);
            
             // Obter o valor BRL selecionado
            $installments = $order->get_meta('card_installments');
            $selected_installment = $this->get_selected_installment($installments);
            $selected_brl_value = $selected_installment['installmentValue'] ?? $order->get_total();

            // Atualizar o total do pedido
            $this->update_order_total($order, $selected_brl_value);
            
            // Adicionar uma nota ao pedido sobre a atualização do valor
            $order->add_order_note(sprintf(__('Valor do pedido atualizado para R$ %s devido ao parcelamento.', 'brazilpays-plugin'), number_format($selected_brl_value, 2, ',', '.')));

            // Atualizar os metadados do pedido com informações adicionais
            $order->update_meta_data('_brazilpays_installments', $installments);
            $order->update_meta_data('_brazilpays_installment_value', $selected_brl_value);

            $card_number = isset($_POST['card_number']) ? sanitize_text_field($_POST['card_number']) : '';
            $last_four = substr($card_number, -4);
            $order->update_meta_data('card_last_four', $last_four);
            $order->update_meta_data('brazilpays_transaction_id', $response['data']['id']);
            $order->add_order_note(__("Método de pagamento: Cartão de crédito", 'brazilpays-plugin'));
            $order->update_status('wc-processing', __('Aguardando confirmação do pagamento - Processando a Operação', 'brazilpays-plugin'));
            $order->save();

            // Remove cart
            WC()->cart->empty_cart();

            return array(
                'result' => 'success',
                'redirect' => $this->get_return_url($order),
            );
        } else {
            return ['result' => 'fail'];
        }
    }

    private function update_order_total($order, $new_total) {
        global $wpdb;
    
        // Atualiza o total diretamente no banco de dados
        $wpdb->update(
            $wpdb->prefix . 'wc_orders',
            array('total_amount' => $new_total),
            array('id' => $order->get_id()),
            array('%f'),
            array('%d')
        );
    
        // Atualiza o total no objeto do pedido
        $order->set_total($new_total);
    
        // Recalcula os totais
        $order->calculate_totals(false);
    
        // Força a atualização dos metadados
        $order->update_meta_data('_order_total', $new_total);
    
        // Limpa o cache
        wp_cache_delete('order-' . $order->get_id(), 'orders');
    
        // Salva as alterações
        $order->save();
    
        // Dispara uma ação para que outros plugins possam reagir à mudança
        do_action('woocommerce_order_total_updated', $order->get_id(), $new_total);
    }

    /**
     * API request method
     *
     * @param string $method
     * @param string $endpoint
     * @param array $data
     * @return array|WP_Error
     */
    private function api_request($method, $endpoint, $data = null) {
        $args = array(
            'method'  => $method,
            'headers' => array(
                'Authorization' => 'Bearer ' . $this->auth_token,
                'Content-Type'  => 'application/json',
                'Accept'        => 'application/json'
            ),
            'timeout' => 60
        );

        if ($data) {
            $args['body'] = wp_json_encode($data);
        }

        $response = wp_remote_request($this->api_url . $endpoint, $args);

        if (is_wp_error($response)) {
            return new WP_Error('brazilpays_api_error', $response->get_error_message());
        }

        $body = wp_remote_retrieve_body($response);
        $data = json_decode($body, true);

        if (wp_remote_retrieve_response_code($response) !== 200) {
            return new WP_Error('brazilpays_api_error', $data['message'] ?? __('Unknown error occurred', 'brazilpays-plugin'));
        }

        return $data;
    }


    /**
     * Prepare order data for API request
     *
     * @param WC_Order $order
     * @param string $total
     * @param array $cotacao_dolar
     * @return array
     */
    private function prepare_order_data($order, $total, $cotacao_dolar)
    {
        $cpfCnpj = $_POST['card_cpf_cnpj'];
        $cardNumber = $_POST['card_number'];
        $cardName = $_POST['card_name'];
        $cardMonth = $_POST['card_month'];
        $cardYear = $_POST['card_year'];
        $cardCvv = $_POST['card_cvv'];
        $cardInstallments = $_POST['card_installments'];
        $gender = $_POST['card_gender'];
        $birthDate = $_POST['card_birth_date'];

       // Obter o número de parcelas selecionado
        $installments = $order->get_meta('card_installments');

        // Obter o valor em BRL selecionado
        $selected_installment = $this->get_selected_installment($installments);
        $selected_brl_value = $selected_installment['installmentValue'] ?? $total;

        return [
            'profile' => [
                'zipCode' => $order->get_billing_postcode(),
                'streetAddress' => $order->get_billing_address_1(),
                'number' => $order->get_meta('_billing_number') ?: preg_replace("/[^0-9]/", "", $order->get_billing_address_1()),
                'cityName' => $order->get_billing_city(),
                'stateName' => $order->get_billing_state(),
                'stateUf' => $order->get_billing_state(),
                'neighborhood' => 'bairro',
                'complement' => $order->get_billing_address_2(),
                'fullName' => $order->get_formatted_billing_full_name(),
                'email' => $order->get_billing_email(),
                'phone' => $order->get_billing_phone(),
                'cpfOrCnpj' => preg_replace("/[^0-9]/", "", $cpfCnpj),
                'creditCard' => [
                    'cardNumber' => $cardNumber,
                    'holderName' => $cardName,
                    'expireMonth' => $cardMonth,
                    'expireYear' => $cardYear,
                    'cvv' => $cardCvv
                ],
                'gender' => $gender,
                'birthDate' => $birthDate,
            ],
            'exchange' => $cotacao_dolar['cotacao'],
            'usedExchange' => $cotacao_dolar['usedExchange'],
            'baseChargeId' => '',
            'invoice' => $order->get_id(),
            'description' => $order->get_id(),
            'typeCharge' => '2',
            'paymentMethod' => '2',
            'installment' => intval($cardInstallments),
            'valuesUsd' => [
                'netValue' => $this->calculated_usd_value,
            ],
            'ValuesBrl' => [
                'netValue' => $selected_brl_value,
            ],
            'urlWebHook' => ''
        ];
    }

    private function get_selected_installment($selected_installments) {
        $installment_options = $this->get_installment_options($this->get_order_total());
        foreach ($installment_options as $option) {
            if ($option['total'] == $selected_installments) {
                return $option;
            }
        }
        return null;
    }

    /**
     * Output for the order received page.
     *
     * @param int $order_id
     */
    public function thankyou_page($order_id)
    {
        echo '<div style="font-size: 20px;color: #303030;text-align: center;">';
        echo '<h2>' . esc_html__('Transação em análise', 'brazilpays-plugin') . '</h2>';
        echo '<h3>' . esc_html__('Estamos processando a sua transação!', 'brazilpays-plugin') . '</h3>';
        echo '</div>';
    }

    /**
     * Add content to the WC emails.
     *
     * @param WC_Order $order
     * @param bool $sent_to_admin
     * @param bool $plain_text
     */
    public function email_instructions($order, $sent_to_admin, $plain_text = false)
    {
        if ($this->instructions && !$sent_to_admin && $this->id === $order->get_payment_method()) {
            echo wp_kses_post(wpautop(wptexturize($this->instructions)) . PHP_EOL);
        }
    }

    /**
     * Authenticate and get token
     *
     * @return string|null
     */
    public function authToken()
    {
        $cached_token = get_transient('brazilpays_token');
        $token_expiry = get_transient('brazilpays_token_expiry');

        if ($cached_token && $token_expiry && time() < $token_expiry) {
            return $cached_token;
        }

        $url = $this->api_url . '/Merchant/External/Token';

        $response = wp_remote_post($url, [
            'headers' => [
                'merchantCode' => $this->merchant_code,
                'publicKey' => $this->public_key
            ],
            'timeout' => 90
        ]);

        if (is_wp_error($response) || wp_remote_retrieve_response_code($response) !== 200) {
            return null;
        }

        $body = wp_remote_retrieve_body($response);
        $data = json_decode($body, true);
        $token = $data['data']['access_token'] ?? null;
        $expires_in = $data['data']['expires_in'] ?? 3600;

        if ($token) {
            set_transient('brazilpays_token', $token, $expires_in);
            set_transient('brazilpays_token_expiry', time() + $expires_in, $expires_in);
        }

        return $token;
    }

    /**
     * Get dollar exchange rate
     *
     * @param string $token
     * @return array|null
     */
    public function cotarDolar($token)
    {
        $url = $this->api_url . '/Charge/Calculate';

        $args = array(
            'headers' => array(
                'Authorization' => 'Bearer ' . $token,
                'Content-Type' => 'application/json'
            ),
            'body' => json_encode(array('amount' => 1.00)),
            'timeout' => 90
        );

        $response = wp_remote_post($url, $args);

        if (is_wp_error($response) || wp_remote_retrieve_response_code($response) != 200) {
            return null;
        }

        $body = wp_remote_retrieve_body($response);
        $data_request = json_decode($body, true);

        return [
            'cotacao' => $data_request['data']['exchange'] ?? null,
            'usedExchange' => $data_request['data']['usedExchange'] ?? null
        ];
    }


    /**
     * Process refunds
     * 
     * @param int $order_id
     * @param float $amount
     * @param string $reason
     * @return bool|WP_Error
     */
    public function process_refund($order_id, $amount = null, $reason = '')
    {
        $order = wc_get_order($order_id);

        if (!$order) {
            return new WP_Error('invalid_order', __('Pedido inválido para reembolso.', 'brazilpays-plugin'));
        }

        $transaction_id = $order->get_meta('id_transacao');

        if (!$transaction_id) {
            return new WP_Error('no_transaction', __('Nenhum ID de transação encontrado para este pedido.', 'brazilpays-plugin'));
        }

        $token = $this->authToken();

        if (!$token) {
            return new WP_Error('auth_error', __('Falha na autenticação com a API BrazilPays.', 'brazilpays-plugin'));
        }

        $refund_data = array(
            'transactionId' => $transaction_id,
            'amount' => $amount,
            'reason' => $reason
        );

        $response = wp_remote_post($this->api_url . '/Refund', array(
            'headers' => array(
                'Authorization' => 'Bearer ' . $token,
                'Content-Type' => 'application/json'
            ),
            'body' => json_encode($refund_data),
            'timeout' => 60
        ));

        if (is_wp_error($response)) {
            return new WP_Error('refund_error', $response->get_error_message());
        }

        $body = wp_remote_retrieve_body($response);
        $result = json_decode($body, true);

        if (isset($result['success']) && $result['success']) {
            $order->add_order_note(sprintf(__('Reembolso de %s processado via BrazilPays.', 'brazilpays-plugin'), wc_price($amount)));
            return true;
        } else {
            return new WP_Error('refund_failed', __('O reembolso falhou. Motivo: ', 'brazilpays-plugin') . ($result['message'] ?? 'Desconhecido'));
        }
    }

    /**
     * Add payment fields on the checkout page
     *
     * @param string $description
     * @param string $payment_id
     * @return string
     */
    public function payment_fields()
{
    $description = $this->get_description();
    if ($description) {
        echo wpautop(wptexturize($description));
    }

    $total_amount = WC()->cart->get_total('edit');   
    //$total_amount = WC()->cart->total;
    $parcelas = $this->get_installment_options($total_amount);

    echo '<div class="brazilpays-credit-form">';
    
    // Campo de parcelas (combobox)
    echo '<div class="brazilpays-installments">';
    woocommerce_form_field('card_installments', array(
        'type' => 'select',
        'class' => array('form-row-wide'),
        'label' => __('Número de Parcelas', 'brazilpays-plugin'),
        'required' => true,
        'options' => array('' => __('Número de parcelas', 'brazilpays-plugin')) + 
                     array_reduce($parcelas, function($carry, $parcela) {
                         if (!empty($parcela['amount']) && !empty($parcela['qtInstallment'])) {
                             $qntInt = (int)$parcela['qtInstallment'];
                             $valor = number_format((float)($parcela['amount']), 2, ',', '.');
                             $carry[$qntInt] = "{$qntInt}x de R$ {$valor}";
                         }
                         return $carry;
                     }, array())
    ));
    echo '</div>';

    // Campo do número do cartão
    woocommerce_form_field('card_number', array(
        'type' => 'text',
        'class' => array('form-row-wide'),
        'label' => __('Número do cartão', 'brazilpays-plugin'),
        'required' => true,
        'custom_attributes' => array(
            'autocomplete' => 'off',
            'pattern' => '\d*',
            'maxlength' => '16'
        )
    ));

    // Campo do nome no cartão
    woocommerce_form_field('card_name', array(
        'type' => 'text',
        'class' => array('form-row-wide'),
        'label' => __('Nome no cartão', 'brazilpays-plugin'),
        'required' => true,
    ));

    // Campos de vencimento do cartão
    echo '<div class="brazilpays-card-expiry">';
    woocommerce_form_field('card_month', array(
        'type' => 'select',
        'class' => array('form-row-first'),
        'label' => __('Mês de vencimento', 'brazilpays-plugin'),
        'required' => true,
        'options' => array_combine(range(1, 12), range('01', '12'))
    ));

    woocommerce_form_field('card_year', array(
        'type' => 'select',
        'class' => 'form-row-last',
        'label' => __('Ano de vencimento', 'brazilpays-plugin'),
        'required' => true,
        'options' => array_combine(range(date('Y'), date('Y') + 15), range(date('Y'), date('Y') + 15))
    ));
    echo '</div>';

    // Campo CVV
    woocommerce_form_field('card_cvv', array(
        'type' => 'number',
        'class' => array('form-row-wide'),
        'label' => __('CVV', 'brazilpays-plugin'),
        'required' => true,
        'custom_attributes' => array(
            'autocomplete' => 'off',
            'maxlength' => '4'
        )
    ));

    // Campo CPF/CNPJ
    woocommerce_form_field('card_cpf_cnpj', array(
        'type' => 'text',
        'class' => array('form-row-wide'),
        'label' => __('CPF ou CNPJ', 'brazilpays-plugin'),
        'required' => true,
        'custom_attributes' => array(
            'pattern' => '\d*',
            'maxlength' => '14'
        )
    ));

    // Campo Gênero
    woocommerce_form_field('card_gender', array(
        'type' => 'select',
        'class' => array('form-row-wide'),
        'label' => __('Gênero', 'brazilpays-plugin'),
        'required' => true,
        'options' => array(
            '' => __('Selecione o gênero', 'brazilpays-plugin'),
            'M' => __('Masculino', 'brazilpays-plugin'),
            'F' => __('Feminino', 'brazilpays-plugin')
        )
    ));

    // Campo Data de Nascimento
    woocommerce_form_field('card_birth_date', array(
        'type' => 'text',
        'class' => array('form-row-wide'),
        'label' => __('Data de nascimento (DD/MM/AAAA)', 'brazilpays-plugin'),
        'required' => true,
        'custom_attributes' => array(
            'pattern' => '\d{2}/\d{2}/\d{4}',
            'placeholder' => 'DD/MM/AAAA'
        )
    ));

    echo '</div>';

    // Script para formatação da data de nascimento
    ?>
    <script>
    jQuery(document).ready(function($) {
        $('#card_birth_date').on('input', function() {
            var value = $(this).val();
            if (value.length === 2 || value.length === 5) {
                $(this).val(value + '/');
            }
        });
    });

    jQuery(document).ready(function($) {
    $('#card_birth_date').on('input', function(e) {
            var input = $(this);
            var value = input.val();
            
            // Remove todos os caracteres não numéricos
            value = value.replace(/\D/g, '');
            
            // Aplica a máscara DD/MM/AAAA
            if (value.length > 2 && value.length <= 4) {
                value = value.replace(/^(\d{2})/, '$1/');
            } else if (value.length > 4) {
                value = value.replace(/^(\d{2})(\d{2})/, '$1/$2/');
            }
            
            // Limita o comprimento total a 10 caracteres (DD/MM/AAAA)
            if (value.length > 10) {
                value = value.substr(0, 10);
            }
        
            input.val(value);
        });
    });

    jQuery(document).ready(function($) {
        $('#card_cpf_cnpj').on('input', function(e) {
            var doc = $(this).val().replace(/\D/g, '');
            if (doc.length <= 11) { // CPF
                doc = doc.replace(/(\d{3})(\d)/, "$1.$2");
                doc = doc.replace(/(\d{3})(\d)/, "$1.$2");
                doc = doc.replace(/(\d{3})(\d{1,2})$/, "$1-$2");
            } else { // CNPJ
                doc = doc.replace(/^(\d{2})(\d)/, "$1.$2");
                doc = doc.replace(/^(\d{2})\.(\d{3})(\d)/, "$1.$2.$3");
                doc = doc.replace(/\.(\d{3})(\d)/, ".$1/$2");
                doc = doc.replace(/(\d{4})(\d)/, "$1-$2");
            }
            $(this).val(doc);
        });
    });
    </script>
    <?php
}

    /**
     * Validate payment fields on the checkout page
     */
    public function validate_fields()
    {
        // Número do cartão
        $card_number = isset($_POST['card_number']) ? trim($_POST['card_number']) : '';
        if (empty($card_number)) {
            wc_add_notice(__('Número do cartão é obrigatório!', 'brazilpays-plugin'), 'error');
            return false;
        }

        // Mês de validade
        $card_month = isset($_POST['card_month']) ? trim($_POST['card_month']) : '';
        if (empty($card_month)) {
            wc_add_notice(__('Mês de validade do cartão é obrigatório!', 'brazilpays-plugin'), 'error');
            return false;
        }

        // Ano de validade
        $card_year = isset($_POST['card_year']) ? trim($_POST['card_year']) : '';
        if (empty($card_year)) {
            wc_add_notice(__('Ano de validade do cartão é obrigatório!', 'brazilpays-plugin'), 'error');
            return false;
        }

        // Verificação da data de validade
        $current_year = date('Y');
        $current_month = date('m');
        if ($card_year < $current_year || ($card_year == $current_year && $card_month < $current_month)) {
            wc_add_notice(__('Data de validade do cartão é inválida!', 'brazilpays-plugin'), 'error');
            return false;
        }

        // CVV
        $card_cvv = isset($_POST['card_cvv']) ? trim($_POST['card_cvv']) : '';
        if (empty($card_cvv) || strlen($card_cvv) < 3 || strlen($card_cvv) > 4) {
            wc_add_notice(__('CVV inválido!', 'brazilpays-plugin'), 'error');
            return false;
        }

         // CPF ou CNPJ
        $card_cpf_cnpj = isset($_POST['card_cpf_cnpj']) ? preg_replace('/[^0-9]/', '', $_POST['card_cpf_cnpj']) : '';
        if (empty($card_cpf_cnpj)) {
            wc_add_notice(__('CPF ou CNPJ é obrigatório!', 'brazilpays-plugin'), 'error');
            return false;
        }
        
        if (!$this->validate_cpf_cnpj($card_cpf_cnpj)) {
            wc_add_notice(__('CPF ou CNPJ inválido!', 'brazilpays-plugin'), 'error');
            error_log('CPF/CNPJ inválido: ' . $card_cpf_cnpj); // Log para depuração
            return false;
        }
    
        // Gênero
        $card_gender = isset($_POST['card_gender']) ? trim($_POST['card_gender']) : '';
        if (empty($card_gender) || !in_array($card_gender, ['M', 'F'])) {
            wc_add_notice(__('Gênero inválido!', 'brazilpays-plugin'), 'error');
            return false;
        }

        // Data de Nascimento
        $card_birth_date = isset($_POST['card_birth_date']) ? trim($_POST['card_birth_date']) : '';
        if (empty($card_birth_date)) {
            wc_add_notice(__('Data de nascimento é obrigatória!', 'brazilpays-plugin'), 'error');
            return false;
        }
        
        // Validação da data de nascimento no formato dd/mm/yyyy
        $date_parts = explode('/', $card_birth_date);
        if (count($date_parts) !== 3) {
            wc_add_notice(__('Formato de data de nascimento inválido. Use DD/MM/AAAA.', 'brazilpays-plugin'), 'error');
            return false;
        }
        
        $day = intval($date_parts[0]);
        $month = intval($date_parts[1]);
        $year = intval($date_parts[2]);
        
        if (!checkdate($month, $day, $year)) {
            wc_add_notice(__('Data de nascimento inválida!', 'brazilpays-plugin'), 'error');
            return false;
        }
        
        $birth_date = DateTime::createFromFormat('d/m/Y', $card_birth_date);
        $current_date = new DateTime();
        if ($birth_date > $current_date) {
            wc_add_notice(__('A data de nascimento não pode ser no futuro!', 'brazilpays-plugin'), 'error');
            return false;
        }
        
        // Verificar se a pessoa tem pelo menos 18 anos
        $age = $current_date->diff($birth_date)->y;
        if ($age < 18) {
            wc_add_notice(__('Você deve ter pelo menos 18 anos para fazer esta compra.', 'brazilpays-plugin'), 'error');
            return false;
        }

        return true;
    }

    
    private function validate_cpf_cnpj($document) {
        $document = preg_replace('/[^0-9]/', '', $document);
        
        if (strlen($document) === 11) {
            return $this->validate_cpf($document);
        } elseif (strlen($document) === 14) {
            return $this->validate_cnpj($document);
        }
        
        error_log('Documento com tamanho inválido: ' . strlen($document)); // Log para depuração
        return false;
    }
    
    private function validate_cpf($cpf) {
        if (strlen($cpf) != 11 || preg_match('/(\d)\1{10}/', $cpf)) {
            error_log('CPF com formato inválido: ' . $cpf); // Log para depuração
            return false;
        }
        
        for ($t = 9; $t < 11; $t++) {
            for ($d = 0, $c = 0; $c < $t; $c++) {
                $d += $cpf[$c] * (($t + 1) - $c);
            }
            $d = ((10 * $d) % 11) % 10;
            if ($cpf[$c] != $d) {
                error_log('CPF inválido: ' . $cpf); // Log para depuração
                return false;
            }
        }
        
        return true;
    }
    
    private function validate_cnpj($cnpj) {
        if (strlen($cnpj) != 14 || preg_match('/(\d)\1{13}/', $cnpj)) {
            error_log('CNPJ com formato inválido: ' . $cnpj); // Log para depuração
            return false;
        }
        
        $b = [6, 5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2];
        
        for ($i = 0, $n = 0; $i < 12; $n += $cnpj[$i] * $b[++$i]);
        if ($cnpj[12] != ((($n %= 11) < 2) ? 0 : 11 - $n)) {
            error_log('CNPJ inválido (primeiro dígito): ' . $cnpj); // Log para depuração
            return false;
        }
        
        for ($i = 0, $n = 0; $i <= 12; $n += $cnpj[$i] * $b[$i++]);
        if ($cnpj[13] != ((($n %= 11) < 2) ? 0 : 11 - $n)) {
            error_log('CNPJ inválido (segundo dígito): ' . $cnpj); // Log para depuração
            return false;
        }
        
        return true;
    }

    /**
     * Get installment options
     *
     * @param float $total_amount
     * @return array
     */
    public function get_installment_options($total_amount)
    {
        $token = $this->authToken();
        if (!$token) {
            return array();
        }

        $response = wp_remote_post($this->api_url . '/Charge/Calculate', array(
            'headers' => array(
                'Authorization' => 'Bearer ' . $token,
                'Content-Type' => 'application/json'
            ),
            'body' => json_encode(array('amount' => $total_amount)),
            'timeout' => 30
        ));

        if (is_wp_error($response) || wp_remote_retrieve_response_code($response) !== 200) {
            return array();
        }

        $body = wp_remote_retrieve_body($response);
        $data = json_decode($body, true);

        // Armazena o valor USD retornado pela API
        $this->calculated_usd_value = $data['data']['valuesUsd']['netValue'] ?? $total_amount;


        return $data['data']['creditCard']['installments'] ?? array();
    }
}
