<?php

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

/**
 * WC_BrazilPays_Gateway_PixNew
 *
 * Provides a PIX payment gateway for BrazilPays with HPOS compatibility
 *
 * @class       WC_BrazilPays_Gateway_PixNew
 * @extends     WC_Payment_Gateway
 * @version     3.0.0
 * @package     WooCommerce\Classes\Payment
 */

 require_once plugin_dir_path(__FILE__) . 'class-brazilpays-environment.php';
 
class WC_BrazilPays_Gateway_PixNew extends WC_Payment_Gateway {
    /**
     * API URL
     *
     * @var string
     */
    private $api_url;

    /**
     * Authentication token
     *
     * @var string
     */
    

    /**
     * Exchange rate
     *
     * @var array
     */
    private $exchange_rate;
    protected $merchant_code;
    protected $public_key;
    protected $auth_token;
    public $description;
    public $instructions;



    /**
     * Constructor for the gateway.
     */
    public function __construct() {

        $this->id                 = 'brazilpays-pix';
        $this->icon               = apply_filters('woocommerce_brazilpays_icon', plugins_url('../assets/icon-pix.png', __FILE__));
        $this->has_fields         = true;
        $this->method_title       = __('PIX - Brazil Pays Pagamentos', 'brazilpays-plugin');
        $this->method_description = __('Aceitar pagamento por PIX pela BrazilPays.', 'brazilpays-plugin');

        $this->init_form_fields();
        $this->init_settings();
 
        $this->title        = $this->get_option('title');
        $this->description  = $this->get_option('description');
        $this->instructions = $this->get_option('instructions', $this->description);

       // Obtem as configurações do BrazilPaysEnvironment
       $brazilpays_environment = BrazilPaysEnvironment::getInstance();
       $this->merchant_code = $brazilpays_environment->getMerchantCode();
       $this->public_key = $brazilpays_environment->getPublicKey();
       $this->api_url = $brazilpays_environment->getApiUrl();

        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        add_action('woocommerce_thankyou_' . $this->id, array($this, 'thankyou_page'));
        add_action('woocommerce_email_before_order_table', array($this, 'email_instructions'), 10, 3);
        //add_action('woocommerce_checkout_process', array($this, 'validate_fields_pix'));
       // add_action('woocommerce_scheduled_subscription_payment_' . $this->id, array($this, 'scheduled_subscription_payment'), 10, 2);

        // HPOS compatibility
        add_action('before_woocommerce_init', array($this, 'declare_hpos_compatibility'));

        // Payment status check
        add_action('woocommerce_order_status_pending_to_processing', array($this, 'check_payment_status'));
        add_action('woocommerce_order_status_pending_to_completed', array($this, 'check_payment_status'));
        add_action('woocommerce_order_status_failed_to_processing', array($this, 'check_payment_status'));


        //$this->api_url = BrazilPaysEnvironment::getInstance()->getApiUrl();
        $this->auth_token = $this->get_auth_token();
       
    }

    /**
     * Declare HPOS compatibility
     */
    public function declare_hpos_compatibility() {
        if (class_exists('\Automattic\WooCommerce\Utilities\FeaturesUtil')) {
            \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility('custom_order_tables', __FILE__, true);
        }
    }


    /**
     * Check payment status
     *
     * @param int $order_id
     */
    public function check_payment_status($order_id) {
        $order = wc_get_order($order_id);
        if (!$order || $order->get_payment_method() !== $this->id) {
            return;
        }

        $transaction_id = $order->get_meta('brazilpays_transaction_id');
        if (!$transaction_id) {
            return;
        }

        $response = $this->api_request('GET', '/Charge/' . $transaction_id);
        if (is_wp_error($response)) {
            $order->add_order_note(__('Failed to check payment status: ', 'brazilpays-plugin') . $response->get_error_message());
            return;
        }

        $this->update_order_status($order, $response['data']['paymentStatus']);
    }

    /**
     * Update order status based on payment status
     *
     * @param WC_Order $order
     * @param int $payment_status
     */
    private function update_order_status($order, $payment_status) {
        switch ($payment_status) {
            case 0: // Pending
                $order->update_status('pending', __('Payment is still pending.', 'brazilpays-plugin'));
                break;
            case 1: // Completed
                $order->payment_complete();
                break;
            case 2: // Failed
            case 3: // Cancelled
                $order->update_status('failed', __('Payment has failed or been cancelled.', 'brazilpays-plugin'));
                break;
            case 4: // Refunded
                $order->update_status('refunded', __('Payment has been refunded.', 'brazilpays-plugin'));
                break;
            default:
                $order->add_order_note(__('Unknown payment status received: ', 'brazilpays-plugin') . $payment_status);
        }
    }

    /**
     * Process the payment and return the result
     *
     * @param int $order_id
     * @return array
     */
    public function process_payment($order_id) {

        // Primeiro, validamos os campos
        $validation_result = $this->validate_fields_pix();
        if (!$validation_result) {
            // Se a validação falhar, retornamos falha
            return array('result' => 'fail');
        }

        $order = wc_get_order($order_id);

        $this->exchange_rate = $this->auth_token ? $this->get_exchange_rate($this->auth_token) : null;

        if (empty($this->auth_token) || empty($this->exchange_rate)) {
            wc_add_notice(__('Error processing payment. Please try again.', 'brazilpays-plugin'), 'error');
            return array('result' => 'fail');
        }

        $total = $order->get_total();

        if ($total < 1.00) {
            wc_add_notice(__('The total order amount must be greater than $1.00!', 'brazilpays-plugin'), 'error');
            return array('result' => 'fail');
        }

        // Captura o CPF/CNPJ do formulário
        $cpf_cnpj = preg_replace('/[^0-9]/', '', $_POST['cpf_cnpj_pix']);
        $order->update_meta_data('cpf_cnpj_pix', $cpf_cnpj);
        $order->save();

        $payment_data = $this->prepare_payment_data($order, $total);
        $response = $this->api_request('POST', '/Charge', $payment_data);

        if (is_wp_error($response)) {
            wc_add_notice($response->get_error_message(), 'error');
            return array('result' => 'fail');
        }

        $this->update_order_meta($order, $response['data']);

        $order->update_status('pending', __('Awaiting PIX payment', 'brazilpays-plugin'));
        WC()->cart->empty_cart();

        return array(
            'result'   => 'success',
            'redirect' => $this->get_return_url($order),
        );
    }

    /**
     * API request method
     *
     * @param string $method
     * @param string $endpoint
     * @param array $data
     * @return array|WP_Error
     */
    private function api_request($method, $endpoint, $data = null) {
        $args = array(
            'method'  => $method,
            'headers' => array(
                'Authorization' => 'Bearer ' . $this->auth_token,
                'Content-Type'  => 'application/json',
                'Accept'        => 'application/json'
            ),
            'timeout' => 60
        );

        if ($data) {
            $args['body'] = wp_json_encode($data);
        }

        $response = wp_remote_request($this->api_url . $endpoint, $args);

        if (is_wp_error($response)) {
            return new WP_Error('brazilpays_api_error', $response->get_error_message());
        }

        $body = wp_remote_retrieve_body($response);
        $data = json_decode($body, true);

        if (wp_remote_retrieve_response_code($response) !== 200) {
            return new WP_Error('brazilpays_api_error', $data['message'] ?? __('Unknown error occurred', 'brazilpays-plugin'));
        }

        return $data;
    }

    /**
     * Update order meta with payment information
     *
     * @param WC_Order $order
     * @param array $payment_data
     */
    private function update_order_meta($order, $payment_data) {
        $order->update_meta_data('brazilpays_transaction_id', $payment_data['id']);
        $order->update_meta_data('brazilpays_pix_code', $payment_data['qrCode']);
        $order->update_meta_data('brazilpays_pix_qr_image', $payment_data['qrCodeImage']);
        $order->update_status('wc-processing', __('Aguardando confirmação do pagamento - Processando a Operação', 'brazilpays-plugin'));
        $order->save();
    }

    /**
     * Process refund
     *
     * @param int $order_id
     * @param float $amount
     * @param string $reason
     * @return bool|WP_Error
     */
    public function process_refund($order_id, $amount = null, $reason = '') {
        $order = wc_get_order($order_id);

        if (!$order) {
            return new WP_Error('invalid_order', __('Invalid order ID', 'brazilpays-plugin'));
        }

        $transaction_id = $order->get_meta('brazilpays_transaction_id');

        if (!$transaction_id) {
            return new WP_Error('no_transaction_id', __('No BrazilPays transaction ID found for this order', 'brazilpays-plugin'));
        }

        $response = $this->api_request('POST', '/Refund', array(
            'transactionId' => $transaction_id,
            'amount' => $amount,
            'reason' => $reason
        ));

        if (is_wp_error($response)) {
            return $response;
        }

        $order->add_order_note(
            sprintf(__('Refunded %1R$s via BrazilPays - Refund ID: %2R$s', 'brazilpays-plugin'),
                wc_price($amount),
                $response['data']['refundId'] ?? 'N/A'
            )
        );

        return true;
    }

    
        /**
         * Display payment fields on the checkout page
         */
        public function payment_fields() {
            echo wpautop(wp_kses_post($this->description));
    
            $total_amount = WC()->cart->get_total('edit');
            //$pix_value = $this->get_pix_value($total_amount);
            $pix_values = $this->get_pix_value($total_amount);

            $pix_value = $pix_values['brl'];
    
            echo '<div class="brazilpays-pix-fields">';
            echo '<p><strong>' . sprintf(__('Pague R$ %s via PIX', 'brazilpays-plugin'), number_format((float)($pix_value), 2, ',', '.')) . '</strong></p>';
            
            
            woocommerce_form_field('cpf_cnpj_pix', array(
                'type' => 'text',
                'label' => __('CPF or CNPJ', 'brazilpays-plugin'),
                'required' => true,
                'class' => array('form-row-wide'),
                'clear' => true,
                'custom_attributes' => array(
                    'maxlength' => '14',
                    'onkeypress' => 'return event.charCode >= 48 && event.charCode <= 57'
                )
            ), WC()->checkout->get_value('cpf_cnpj_pix'));        
            
            echo '</div>';
        }
    
        /**
         * Validate payment fields on the checkout page
         *
         * @return bool
         */
        public function validate_fields_pix() {
            if (empty($_POST['cpf_cnpj_pix'])) {
                wc_add_notice(__('Please enter a valid CPF or CNPJ.', 'brazilpays-plugin'), 'error');
                return false;
            }
    
            $cpf_cnpj = preg_replace('/[^0-9]/', '', $_POST['cpf_cnpj_pix']);
            if (!$this->validate_cpf_cnpj($cpf_cnpj)) {
                wc_add_notice(__('The CPF or CNPJ entered is invalid.', 'brazilpays-plugin'), 'error');
                return false;
            }
    
            return true;
        }
    
        /**
         * Validate CPF or CNPJ
         *
         * @param string $document
         * @return bool
         */
        private function validate_cpf_cnpj($document) {
            $document = preg_replace('/[^0-9]/', '', $document);
            
            if (strlen($document) === 11) {
                return $this->validate_cpf($document);
            } elseif (strlen($document) === 14) {
                return $this->validate_cnpj($document);
            }
            
            return false;
        }
    
        /**
         * Validate CPF
         *
         * @param string $cpf
         * @return bool
         */
        private function validate_cpf($cpf) {
            if (strlen($cpf) != 11 || preg_match('/(\d)\1{10}/', $cpf)) {
                return false;
            }
            
            for ($t = 9; $t < 11; $t++) {
                for ($d = 0, $c = 0; $c < $t; $c++) {
                    $d += $cpf[$c] * (($t + 1) - $c);
                }
                $d = ((10 * $d) % 11) % 10;
                if ($cpf[$c] != $d) {
                    return false;
                }
            }
            
            return true;
        }
    
        /**
         * Validate CNPJ
         *
         * @param string $cnpj
         * @return bool
         */
        private function validate_cnpj($cnpj) {
            if (strlen($cnpj) != 14 || preg_match('/(\d)\1{13}/', $cnpj)) {
                return false;
            }
            
            $b = [6, 5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2];
            
            for ($i = 0, $n = 0; $i < 12; $n += $cnpj[$i] * $b[++$i]);
            if ($cnpj[12] != ((($n %= 11) < 2) ? 0 : 11 - $n)) {
                return false;
            }
            
            for ($i = 0, $n = 0; $i <= 12; $n += $cnpj[$i] * $b[$i++]);
            if ($cnpj[13] != ((($n %= 11) < 2) ? 0 : 11 - $n)) {
                return false;
            }
            
            return true;
        }
    
        /**
         * Get PIX value based on total amount
         *
         * @param float $total_amount
         * @return float
         */
        private function get_pix_value($total_amount) {
            if (empty($this->auth_token)) {
                return array('usd' => $total_amount, 'brl' => $total_amount);
            }
    
            $response = $this->api_request('POST', '/Charge/Calculate', array('amount' => $total_amount));
    
            if (is_wp_error($response)) {
                return array('usd' => $total_amount, 'brl' => $total_amount);
            }

            // Verifica se a resposta contém os dados esperados
            if (isset($response['data']['pix']['grossValue'])) {
                $usd_value = $response['data']['valueUSD'];
                $brl_value = $response['data']['pix']['grossValue'] ?? $usd_value;
            } else {
                $usd_value = $total_amount;
                $brl_value = $total_amount;
            }

            return array('usd' => $usd_value, 'brl' => $brl_value);
    
            //return $response['data']['pix']['grossValue'] ?? $total_amount;
            
        }
    
        /**
         * Output for the order received page.
         *
         * @param int $order_id
         */
        public function thankyou_page($order_id) {
            $order = wc_get_order($order_id);
            $pix_code = $order->get_meta('brazilpays_pix_code');
            $pix_qr_image = $order->get_meta('brazilpays_pix_qr_image');
    
            echo '<h2>' . esc_html__('PIX Payment', 'brazilpays-plugin') . '</h2>';
            echo '<p>' . esc_html__('Escaneie o Qr Code abaixo para fazer o Pagamento:', 'brazilpays-plugin') . '</p>';
            echo '<img src="' . esc_url($pix_qr_image) . '" alt="PIX QR Code" style="display:block; margin:0 auto; max-width:200px;">';
            echo '<p style="text-align:center;"><strong>' . esc_html__('PIX Code:', 'brazilpays-plugin') . '</strong><br>' . esc_html($pix_code) . '</p>';
        }
    
        /**
         * Add content to the WC emails.
         *
         * @param WC_Order $order
         * @param bool $sent_to_admin
         * @param bool $plain_text
         */
        public function email_instructions($order, $sent_to_admin, $plain_text = false) {
            if ($this->instructions && !$sent_to_admin && $this->id === $order->get_payment_method() && $order->has_status('pending')) {
                echo wp_kses_post(wpautop(wptexturize($this->instructions)) . PHP_EOL);
            }
        }
    
        /**
         * Process scheduled subscription payments
         *
         * @param float $amount_to_charge
         * @param WC_Order $renewal_order
         */
        public function scheduled_subscription_payment($amount_to_charge, $renewal_order) {
            // Implementation for processing scheduled subscription payments
            // This will depend on how BrazilPays handles recurring payments for PIX
            // You may need to generate a new PIX code for each renewal
            
            $response = $this->api_request('POST', '/SubscriptionRenewal', array(
                'orderId' => $renewal_order->get_id(),
                'amount' => $amount_to_charge,
                // Add any other required parameters
            ));
    
            if (is_wp_error($response)) {
                $renewal_order->update_status('failed', __('Subscription renewal failed: ', 'brazilpays-plugin') . $response->get_error_message());
                return;
            }
    
            // Process the response and update the renewal order
            $this->update_order_meta($renewal_order, $response['data']);
            $renewal_order->update_status('pending', __('Awaiting PIX payment for subscription renewal', 'brazilpays-plugin'));
        }
    
        /**
         * Prepare payment data for API request
         *
         * @param WC_Order $order
         * @param float $total
         * @return array
         */
        private function prepare_payment_data($order, $total) {

            $total_amount = WC()->cart->get_total('edit');
            $pix_values = $this->get_pix_value($total_amount);

            return array(
                'profile' => array(
                    'zipCode'       => $order->get_billing_postcode(),
                    'streetAddress' => $order->get_billing_address_1(),
                    'number'        => $order->get_meta('_billing_number') ?: preg_replace("/[^0-9]/", "", $order->get_billing_address_1()),
                    'cityName'      => $order->get_billing_city(),
                    'stateName'     => $order->get_billing_state(),
                    'stateUf'       => $order->get_billing_state(),
                    'neighborhood'  => $order->get_meta('_billing_neighborhood') ?: 'N/A',
                    'complement'    => $order->get_billing_address_2(),
                    'fullName'      => $order->get_formatted_billing_full_name(),
                    'email'         => $order->get_billing_email(),
                    'phone'         => $order->get_billing_phone(),
                    'cpfOrCnpj'     => preg_replace("/[^0-9]/", "", $order->get_meta('cpf_cnpj_pix')),
                    'gender' => 'O',
                    'birthDate' => '20/04/1977'
                ),
                'exchange'      => $this->exchange_rate['cotacao'],
                'usedExchange'  => $this->exchange_rate['usedExchange'],
                'invoice'       => $order->get_id(),
                'description'   => sprintf(__('Order %s', 'brazilpays-plugin'), $order->get_order_number()),
                'typeCharge'    => '0', // PIX
                'paymentMethod' => '0', // PIX
                'valuesUsd'     => array(
                    'netValue' => $pix_values['usd'],
                ),
                'ValuesBrl' => [
                    'netValue' => $total,
                ]
            );
        }
    
        /**
         * Get authentication token
         *
         * @return string|null
         */
        private function get_auth_token() {
            $cached_token = get_transient('brazilpays_token');
            if ($cached_token) {
                return $cached_token;
            }
    
            $response = wp_remote_post($this->api_url . '/Merchant/External/Token', array(
                'headers' => array(
                    'merchantCode' => $this->merchant_code,
                    'publicKey' => $this->public_key
                ),
                'timeout' => 30
            ));
    
            if (is_wp_error($response) || wp_remote_retrieve_response_code($response) !== 200) {
                return null;
            }
    
            $body = wp_remote_retrieve_body($response);
            $data = json_decode($body, true);
            $token = $data['data']['access_token'] ?? null;
            $expires_in = $data['data']['expires_in'] ?? 3600;
    
            if ($token) {
                set_transient('brazilpays_token', $token, $expires_in);
            }
    
            return $token;
        }
    
        /**
         * Get exchange rate
         *
         * @param string $token
         * @return array|null
         */
        private function get_exchange_rate($token) {
            $response = $this->api_request('POST', '/Charge/Calculate', array('amount' => 1.00));
    
            if (is_wp_error($response)) {
                return null;
            }
    
            return array(
                'cotacao' => $response['data']['exchange'] ?? null,
                'usedExchange' => $response['data']['usedExchange'] ?? null
            );
        }
    }